{- This file is part of file4life, a file backup program written in Haskell.
 - file4life is Copyright (C) Antonio Bellezza 2014
 - file4life is free software: you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation, either version 3 of the License, or
 - (at your option) any later version.

 - file4life is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.

 - You should have received a copy of the GNU General Public License
 - along with file4life.  If not, see <http://www.gnu.org/licenses/>.
 -}
{-# LANGUAGE RankNTypes #-}
module F4L.PathAttributes where

import F4L
import F4L.Utils( pathToShort, ShortPath )

import Control.DeepSeq
import Control.Lens
import Control.Monad( mplus )
import Data.Binary
import qualified Data.ByteString.Short as SBS
import Data.Functor.Identity
import qualified Data.Map.Strict as SM
import System.FilePath( splitDirectories )


data PathNode a = PathNode {
    _pn_val :: !a
  , _pn_children :: !( SM.Map SBS.ShortByteString ( PathNode a ) )
  } deriving ( Show )


empty = PathNode Nothing SM.empty


pathSet :: FilePath -> Setter' ( PathNode ( Maybe a ) ) ( PathNode ( Maybe a ) )
pathSet fp = sets $ pathSet' ps
  where
    ps = map pathToShort $ splitDirectories fp
    pathSet' :: [ ShortPath ] -> ( PathNode ( Maybe a ) -> PathNode ( Maybe a ) ) -> PathNode ( Maybe a ) -> PathNode ( Maybe a )
    pathSet' [] f x = f x
    pathSet' ( p : ps ) f ( PathNode a m ) =
      PathNode a $ SM.alter ( Just . maybe ( pathSet' ps f ( PathNode Nothing SM.empty ) ) ( pathSet' ps f ) ) p m


pathLens :: Bool -> FilePath -> Lens' ( PathNode ( Maybe a ) ) ( Maybe a )
pathLens inherit fp = lens ( get ps ) ( set ps )
  where
    ps = map pathToShort $ splitDirectories fp
    get :: [ ShortPath ] -> PathNode ( Maybe a ) -> Maybe a
    get = get' Nothing
    get' pre [] ( PathNode a _ ) = a `mplus1` pre
    get' pre ( p : ps ) ( PathNode a m ) = ( SM.lookup p m >>= get' Nothing ps ) `mplus1` a `mplus1` pre
    mplus1 = if inherit then mplus else const
    set :: [ ShortPath ] -> PathNode ( Maybe a ) -> Maybe a -> PathNode ( Maybe a )
    set [] ( PathNode _ m ) v = PathNode v m
    set ( p : ps ) ( PathNode a m ) v =
      PathNode a $ SM.alter ( Just . maybe ( set ps ( PathNode Nothing SM.empty ) v ) ( flip ( set ps ) v ) ) p m


instance NFData a => NFData ( PathNode a ) where
  rnf x @ ( PathNode a m ) = rnf a `seq` rnf m


instance ( Binary a, NFData a ) => Binary ( PathNode a ) where
  put ( PathNode a m ) = a `seq` m `seq` put a >> put m
  get = do
        a <- get
        m <- get
        return $!! PathNode a m
